/** Soluzione del Laboratorio di Algoritmica dell'11/01/10 --aorlandi
Nota: esistono soluzioni piu' compatte ma meno facili da commentare,
per cui ho scelto quella un po' piu' lunga. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

/* Lunghezza massima. */
#define MAXLEN 30
/* Dimensione massima dell'alfabeto (solo minuscole) */
#define ALFASIZE 26


/* Struttura che contiene ogni stringa
   dato in input oltre ai relativi conteggi. */
typedef struct {
   /* Data l'anagramma principale della stringa
      in input sia X l'i-esimo tipo di simbolo
      in essa contenuto: conteggi[i] 
      contiene il numero di X all'interno della
   */
   int conteggi[ALFASIZE];
   /* Una stringa in input. */
   char *stringa;
} entry; 

/* Confrontatore da usare con qsort()
   di sistema. Usato per ordinare
   tutte le stringhe contemporaneamente:
   la stringa A e' minore di B se
   viene in un gruppo precedente oppure
   se viene prima di B all'interno dello
   stesso gruppo.
   Il primo predicato viene controllato
   considerando i conteggi, e solo
   in caso di uguaglianza di conteggi
   si usa l'ordine alfabetico. */
int 
cmpEse (const void *p1, const void *p2)
{
    int i;

    entry *e1, *e2;
    e1 = (entry *) p1;
    e2 = (entry *) p2;

    for ( i = 0; i < ALFASIZE ; i++ ) 
    {
       if ( e1->conteggi[i] < e2->conteggi[i] ) 
           return -1;
       else if ( e1->conteggi[i] > e2->conteggi[i] ) 
           return +1;
    }
    return strcmp( e1->stringa, e2->stringa);
}


/* Confrontatore da usare con qsort() che confronta
   due caratteri sull'ordine alfabetico. */
int 
cmpSymbol(const void *p1, const void *p2)
{
   return *(char *) p1 - *(char *)p2;
}

/* Funzione che riceve in input un anagramma
   principale e scrive all'interno di "conteggi"
   (che si suppone almeno ALFASIZE interi)
   i conteggi dei vari caratteri.
   Basta una scansione sequenziale dell'anagramma. */
void conta(char *anagrammaprinc, int *conteggi)
{
  int len = strlen(anagrammaprinc);
  int i, corrente ;

  /* Inizializzazione */
  for ( i = 0; i < ALFASIZE; i++ ) 
     conteggi[i] = 0;

  
  corrente = 0;
  conteggi[corrente] = 1;
  /* Ogni volta che si cambia simbolo,
     cambiamo locazione di conteggio. */
  for ( i = 1; i < len; i++ )
  {
     if ( anagrammaprinc[i-1] != anagrammaprinc[i] )
         corrente++;
     conteggi[corrente]++;     
  }
}

/* Crea l'anagramma princpale della stringa "stringa"
   in "anagrammaprinc" */
void anagramma(char *stringa, char *anagrammaprinc) 
{
    /* Copia e usa qsort() sui caratteri per anagrammare. */
   strcpy ( anagrammaprinc, stringa );
   qsort ( anagrammaprinc,  strlen(anagrammaprinc), sizeof(char), cmpSymbol);
}

/* Funzione che ritorna 1 se due conteggi
   sono uguali, 0 altrimenti. */
int stessoGruppo ( entry e1, entry e2 )
{
   int i;
   for ( i = 0; i < ALFASIZE; i++ ) 
     if ( e1.conteggi[i] != e2.conteggi[i] ) 
        return 0; 
   return 1;
}

/* Legge l'input e crea i conteggi. */
entry * leggi(int *n)
{
  entry *tutte;
  char *anagrammaprinc = malloc(sizeof(char)*(MAXLEN+1));
  int i, j;

  scanf("%d", n); 

  tutte = (entry *) malloc(sizeof(entry)* (*n));

  for ( i = 0; i < *n; i++ ) 
  {
     tutte[i].stringa = (char *)malloc(sizeof(char)*(MAXLEN+1));
     scanf("%s", tutte[i].stringa);
     anagramma ( tutte[i].stringa, anagrammaprinc );
     conta( anagrammaprinc, tutte[i].conteggi );
  }

  return tutte;
}


int main()
{
  int n, i;
  entry *coppie;
  
  coppie = leggi(&n); 
  
  /* Dopo la lettura, ordina le stringhe fra di loro
     pronte per l'output */
  qsort (coppie, n, sizeof(entry), cmpEse);

  /* Output */
  printf("%s", coppie[0].stringa);
  for (i = 1; i < n; i++ ) {
    if ( !stessoGruppo( coppie[i], coppie[i-1] ) )
	printf("\n");
    else printf(" ");
    printf("%s", coppie[i].stringa);
  }

  printf("\n");
return 0;
}
